""" DarkSide 20K Construction Database User Interface

    This library is meant to ease the access to the DarkSide 20K
    construction database.

    Refer to the README.md file for further details.
"""

__app__ = "ds20kdb"
__author__ = "Carmelo Pellegrino"
__maintainer__ = "Carmelo Pellegrino"
__copyright__ = (
    "Museo storico della fisica e centro ricerche e studi Enrico Fermi")
__license__ = "EUPL"
__version__ = "0.0.1"
__email__ = "carmelo.pellegrino@centrofermi.it"
__status__ = "Development"

import ds20kdb.auth
import ds20kdb.request
